#include <stdio.h>
#include <stdlib.h>
#include "miniglut.h"
#include <GL/glu.h>

#ifndef GL_MULTISAMPLE
#define GL_MULTISAMPLE	0x809d
#endif

void display(void);
void draw_object(int n);
void reshape(int x, int y);
void keyboard(unsigned char key, int x, int y);
void mouse(int bn, int st, int x, int y);
void motion(int x, int y);

int sel = -1;

float cam_theta, cam_phi = 25, cam_dist = 8;
int bnstate[3], prev_x, prev_y;


int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitWindowSize(1280, 800);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE | GLUT_MULTISAMPLE);
	glutCreateWindow("wireframe over filled");

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);

	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_MULTISAMPLE);

	glutMainLoop();
	return 0;
}

void display(void)
{
	int i;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0, 0, -cam_dist);
	glRotatef(cam_phi, 1, 0, 0);
	glRotatef(cam_theta, 0, 1, 0);

	for(i=0; i<3; i++) {
		if(sel == i) {
			/* draw selected object with a positive polygon offset ... */
			glPolygonOffset(1, 1);
			glEnable(GL_POLYGON_OFFSET_FILL);
			draw_object(i);
			glDisable(GL_POLYGON_OFFSET_FILL);

			/* and then draw it again as wireframe */
			glLineWidth(3);
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

			glDisable(GL_LIGHTING);
			glColor3f(0.1, 1.0, 0.4);
			draw_object(i);

			glEnable(GL_LIGHTING);
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		} else {
			/* draw the rest of the objects normally */
			draw_object(i);
		}
	}

	glutSwapBuffers();
}

void draw_object(int n)
{
	float kd[] = {0.5, 0.6, 0.8, 1.0};
	float ks[] = {0.8, 0.8, 0.8, 1.0};

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, kd);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, ks);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 60.0);

	glPushMatrix();

	switch(n) {
	case 0:
		glFrontFace(GL_CW);
		glutSolidTeapot(1.0);
		glFrontFace(GL_CCW);
		break;

	case 1:
		glTranslatef(-3, 0, 0);
		glRotatef(90, 1, 0, 0);
		glutSolidSphere(1, 24, 12);
		break;

	case 2:
		glTranslatef(3, 0, 0);
		glutSolidCube(1.25);
		break;

	default:
		break;
	}

	glPopMatrix();
}


void reshape(int x, int y)
{
	glViewport(0, 0, x, y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50, (float)x / (float)y, 0.5, 500);
}

void keyboard(unsigned char key, int x, int y)
{
	static const char *objname[] = {"teapot", "sphere", "cube"};

	switch(key) {
	case 27:
		exit(0);

	case '\t':
		sel = (sel + 1) % 3;
		printf("selected: %s\n", objname[sel]);
		glutPostRedisplay();
		break;
	}
}

void mouse(int bn, int st, int x, int y)
{
	int idx = bn - GLUT_LEFT_BUTTON;
	int press = st == GLUT_DOWN;

	if(idx < 3) bnstate[idx] = press;

	prev_x = x;
	prev_y = y;
}

void motion(int x, int y)
{
	int dx = x - prev_x;
	int dy = y - prev_y;
	prev_x = x;
	prev_y = y;

	if((dx | dy) == 0) return;

	if(bnstate[0]) {
		cam_theta += dx * 0.5;
		cam_phi += dy * 0.5;
		if(cam_phi < -90) cam_phi = -90;
		if(cam_phi > 90) cam_phi = 90;
		glutPostRedisplay();
	}
	if(bnstate[2]) {
		cam_dist += dy * 0.1;
		if(cam_dist < 0) cam_dist = 0;
		glutPostRedisplay();
	}
}
